using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

using BearWare;

namespace TeamTalk.NET
{
    class FilesView
    {
        TeamTalk4 ttclient;
        ListView listview;

        public FilesView(TeamTalk4 tt, ListView list)
        {
            this.ttclient = tt;
            this.listview = list;

            listview.Columns.Add("File ID");
            listview.Columns.Add("Filename");
            listview.Columns.Add("Size");
            listview.Columns.Add("Owner");

            ttclient.OnCmdFileNew += new TeamTalk4.FileUpdate(ttclient_OnCmdFileNew);
            ttclient.OnCmdFileRemove += new TeamTalk4.FileUpdate(ttclient_OnCmdFileRemove);
            ttclient.OnCmdMyselfLeftChannel += new TeamTalk4.MyselfLeftChannel(ttclient_OnCmdMyselfLeftChannel);
        }

        public int GetSelectedFile()
        {
            if (listview.SelectedItems.Count == 0)
                return 0;
            return (int)listview.SelectedItems[0].Tag;
        }

        public int GetSelectedChannel()
        {
            return ttclient.GetMyChannelID(); //for now just "my" channel
        }

        void ttclient_OnCmdMyselfLeftChannel(int nChannelID)
        {
            listview.Items.Clear(); //ensure we don't get dublicates if joining again
        }

        void ttclient_OnCmdFileNew(int nFileID, int nChannelID)
        {
            if (ttclient.GetMyChannelID() != nChannelID)
                return;//an admin can see all files but we don't show it here (for now)

            FileInfo file;
            if (!ttclient.GetChannelFileInfo(nChannelID, nFileID, out file))
                return;

            ListViewItem item = new ListViewItem();
            item.Tag = nFileID;
            item.Text = file.nFileID.ToString();
            item.SubItems.Add(file.szFileName);
            item.SubItems.Add(file.nFileSize.ToString());
            item.SubItems.Add(file.szUsername);
            listview.Items.Add(item);
        }

        void ttclient_OnCmdFileRemove(int nFileID, int nChannelID)
        {
            if (ttclient.GetMyChannelID() != nChannelID)
                return;

            foreach (ListViewItem item in listview.Items)
                if ((int)item.Tag == nFileID)
                    listview.Items.Remove(item);
        }
    }
}
